#ifndef EMADevice_H
#define EMADevice_H 

#include <stdint.h>

// Error Return Codes                                             
// 
#define EMA_INVALID_PARAMETER             0x80000001L
#define EMA_INVALID_HANDLE                0x80000002L
#define EMA_OUTOFMEMORY                   0x80000003L
#define EMA_EMPTY_FILE_PATH               0x80000004L
#define EMA_READ_FAULT                    0x80000005L
#define EMA_WRITE_FAULT                   0x80000006L
#define EMA_BUFFER_TOO_SMALL              0x80000007L
#define EMA_WAIT_TIMEOUT                  0x80000008L
#define EMA_NOT_IMPLEMENTED               0x80000009L
#define EMA_HID_CALL_FAILURE              0x8000000AL
#define EMA_READ_DATA_FLAG_FAILURE        0x8000000BL
#define EMA_GET_AD_STD_DATA_FAILURE       0x8000000CL
#define EMA_STARTPOLL_FAILURE             0x8000000DL
#define EMA_NOT_SUPPORTED                 0x8000000EL
#define EMA_DEVICE_IS_NOT_CONNECTED       0x8000000FL
#define EMA_STEREO_MODE_IS_NOT_SUPPORTED  0x80000010L
#define EMA_INVALID_DISPLAY_TYPE          0x80000011L
#define EMA_INVALID_VALUE                 0x80000012L
#define EMA_INVALID_PROPERTY_ID           0x80000013L
#define EMA_ALREADY_ENABLED               0x80000014L
#define EMA_ALREADY_DISABLED              0x80000015L
#define EMA_MOTION_DETECTOR_FAILURE       0x80000016L
#define EMA_NO_DEVICE_FOUND               0x80000017L

// Enumerated Types
// 
enum EMADeviceType
{
    EMA_ALL = 0,
    EMA_Z800,
	EMA_Z800B,
    EMA_X800,
};

enum EMAFlags
{
    EMA_NONE = 0 ,
    EMA_NO_HEADTRACKING = 0x1 ,
};

enum EMADisplayType
{
    EMA_DONT_CARE = 0 ,
    EMA_MONO_DISPLAY,
    EMA_LEFT_DISPLAY,
    EMA_RIGHT_DISPLAY,
    EMA_BOTH_DISPLAY
};

enum EMAProperty
{
    // Eeprom properties 
    EMA_AC_TIMEOUT = 0 ,
    EMA_PHASE ,
    EMA_PIXELCLOCK ,
    EMA_USB_TIMEOUT ,
	EMA_RED_GAIN ,
	EMA_GREEN_GAIN ,
	EMA_BLUE_GAIN ,
	EMA_RED_GAIN_OFFSET ,
	EMA_GREEN_GAIN_OFFSET ,
	EMA_BLUE_GAIN_OFFSET ,
    EMA_ENABLE_EXTERNAL_MONITOR ,
	EMA_HORIZONTAL_POSITION ,
    EMA_VERTICAL_POSITION ,
	EMA_BRIGHTNESS_RAMP_0 ,
	EMA_BRIGHTNESS_RAMP_1 ,
	EMA_BRIGHTNESS_RAMP_2 ,

    // registry properties 
    EMA_MOUSE_TRACKING_SPEED_X , 
	EMA_MOUSE_TRACKING_SPEED_Y , 
	EMA_MOUSE_TRACKING_X_AXIS_DIRECTION , 
	EMA_MOUSE_TRACKING_Y_AXIS_DIRECTION , 

};

// Structures                                              
//

struct EMADeviceCap
{
    unsigned short EMA_HEAD_TRACKER : 1;
    unsigned short EMA_MONOCULAR    : 1;
    unsigned short EMA_BINOCULAR    : 1;
    unsigned short EMA_CANDO3D      : 1;
};

struct EMADeviceData
{
    float AccX ;
    float AccY ;
    float AccZ ;
    float AngX ;
    float AngY ;
    float AngZ ;
    float North ;
    float Yaw ;
    float Pitch ;
    float Roll ;

};

struct EMADeviceInfo
{
    EMADeviceType Type ;
    EMADeviceCap  Capabilities ;

    int        FirmwareVersion ;
	int        DisplayFrequency ;
	int        ScreenX ;
	int        ScreenY ;
    long       VendorID ;
    long       ProductID ;
};

struct EMAGainOffset
{
    uint8_t RedGain ;
    uint8_t GreenGain ;
    uint8_t BlueGain ;
    uint8_t RedOffset ;
    uint8_t GreenOffset ;
    uint8_t BlueOffset ;
};


class EMADevice
{
public:
    EMADevice();
    EMADevice( EMADeviceInfo * deviceInfo );
    EMADevice( EMAFlags flags );
    EMADevice( EMADeviceInfo * deviceInfo, EMAFlags flags );
	~EMADevice();

    static int GetLastError( void );
	static int GetVersion( int * version, int * build );
	static int EnumerateDevices( EMADeviceInfo * devInfos, int maxDevices, int * foundDevices );
    int GetDeviceInfo( EMADeviceInfo * devInfos ); 

    // Mouse and head tracking 
    // -----------------------
    //
    int EnableMouseTracking( bool enable );
	int ResetMouseTracking();
    int PollHeadTrackerRawData( EMADeviceData * deviceData ); 

    // 3D and Misc USB commands 
    // ------------------------
    //
    int EnableExternalMonitor( bool enable );
	int EnableStereoVision( bool enable );
	int GoSleep();
	int KeepAlive();
	int GetFirmwareVersion(uint32_t & version);
	int ToggleLeftRightDisplays();
    int CycleBrightnessLevel();
	int ResetEEPROM();

    // Properties 
    // ----------
    //
	int GetProperty( EMADisplayType display, EMAProperty propertyID , int * propertyValue );
	int SetProperty( EMADisplayType display, EMAProperty propertyID , int propertyValue );
	static int GetPropertyRange( EMAProperty PropertyID, int * minValue , int * maxValue );
	static int GetPropertyFactorySetting( EMAProperty propertyID, int * propertyDefault );
	int RestorePropertyFactorySetting( EMADisplayType display, EMAProperty propertyID );
   	int RestoreFactorySettings();

    // Gain and Offset 
    // ---------------
    //
    int GetGainOffset( EMADisplayType display, EMAGainOffset * data );
	int SetGainOffset( EMADisplayType display, EMAGainOffset * data );
	static int GetGainRange ( uint8_t * minGain, uint8_t * maxGain );
	static int GetOffsetRange ( uint8_t * minOffset, uint8_t * maxOffset );
	static int GetGainOffsetFactorySetting ( uint8_t * defaultGain, uint8_t * defaultOffset );
	int RestoreGainOffsetFactorySetting ( EMADisplayType display );

private:
	int m_device;
	uint32_t m_version;
	
    void init( EMADeviceInfo * deviceInfo, EMAFlags flags ); 
};

#endif // EMADevice_H
